;Easy Lights 1
;By Steven R. Cypherd 5/9/2012
;I am using my PicAxe 28 X2 robot for this example.
;All of my outputs are used for my Servos and Motors.
;X2s can use all ports for almost anything.
;I show you how to use a Serial Shift Register.
;Shift register is the CD74HC164
;	8 bit Serial In to Parallel out.
;	Data is shifted in on the rising edge
;	of the clock pulse. No handshaking.
;	This is electronics chip clocking.
;Serial for the shift registers is run manually in
;doLights sub routine.(No handshaking, Stop or Start bits
;in these devices).
;
;The Picaxe does not support arrays. Only Tables
;loaded in with the program.
;
;I only had Inputs to use.
;Ports and Constants
symbol pClock		=	C.0		;Input/out	0:	Clock
symbol pData		=	C.1		;Input/out	1:	Data
symbol pM_Reset	=	C.2		;Input/out	2:	Master Reset
symbol pBtn1		=	pinC.3	;Input	3:	Button 1
symbol pBtn2		=	pinC.4	;Input	4:	Button 2
symbol Set_Data 	= 1			;Setup time in Microseconds
symbol Set_Clock 	= 1			;Uncomment in doLights

;Variables Bits
symbol lit1			= bit0	;First 4 bytes
symbol lit2			= bit1	;are bit addressable
symbol lit3			= bit2	;Shift register lights
symbol lit4			= bit3
symbol lit5			= bit4
symbol lit6			= bit5
symbol lit7			= bit6
symbol lit8			= bit7
symbol useMreset	= bit8	;SysFlags
symbol backShift	= bit9
symbol modeSet		= bit10
symbol btnDown		= bit11
symbol setSpeed	= bit12
;Bytes
symbol lights		= b0	;lit1 - lit8
symbol sysFlags	= b1
symbol shiftCnt	= b2	;Bit counter
symbol RunMode		= b3
symbol cnt1			= b4
symbol cnt2			= b5
symbol tmp1			= b6
symbol tmp2			= b7
symbol tmpBtn		= b8
symbol litTmp		= b9
symbol litSpeed	= b10
symbol myPause		= b11
symbol myLowTime	= b12	;over lap variables
symbol myHiTime	= b13	;OK here
;Words
symbol myTime		= w6	;Bytes 12 & 13

main:
pause 1000
'This is my 28X2 robot. You can use whatever
'ports you like.
output pClock
output pData
output pM_Reset


backShift = 0
'Using useMreset(Master Reset) fixes flickers
'when clearing the lights. Input 2 connected to MR
useMreset = 1
shiftCnt = 8
cnt1 = 500
cnt2 = 1
modeSet = 1
btnDown = 0
setSpeed = 0
tmpBtn = 0
litSpeed = 4000
RunMode = 1

high pM_Reset

runIt:
'sertxd("lit", #cnt2, 32)
myTime = 63 * cnt1

if btnDown = 0 and pBtn1 = 1 then
	tmpBtn = tmpbtn + 1
	if tmpBtn < 4 then runIt
	RunMode = RunMode + 1
	modeSet = 1
	btnDown = 1
	tmpBtn = 0
	if RunMode > 3 then
		RunMode = 1
		endif
	endif
	
if btnDown = 0 and pBtn2 = 1 then
	tmpBtn = tmpbtn + 1
	if tmpBtn < 4 then runIt
	setSpeed = 1
	modeSet = 1
	btnDown = 1
	tmpBtn = 0
	endif
	
if btnDown = 1 then
	tmpBtn = tmpBtn + 1
	if tmpBtn > 6 then
		btnDown = 0
		endif
	endif

select case RunMode
	case 1
		if modeSet = 1 then
			litSpeed = 40000
			modeSet = 0
			myPause = 2
			if setSpeed = 1 then
				myPause = myPause - 1
				litSpeed = litSpeed - 1000
				setSpeed = 0
				if myPause = 0 then
					myPause = 2
					litSpeed = 40000
					endif
				endif
			endif
		gosub doLarson
	case 2
		if modeSet = 1 then
			litSpeed = 60000
			modeSet = 0
			myPause = 6
			if setSpeed = 1 then
				myPause = myPause - 1
				litSpeed = litSpeed - 1000
				setSpeed = 0
				if myPause < 4 then
					myPause = 6
					litSpeed = 60000
					endif
				endif
			endif
		gosub doBinaryNs
	case 3
		if modeSet = 1 then
			litSpeed = 60000
			modeSet = 0
			myPause = 6
			if setSpeed = 1 then
				myPause = myPause - 1
				litSpeed = litSpeed - 1000
				setSpeed = 0
				if myPause < 4 then
					myPause = 6
					litSpeed = 60000
					endif
				endif
			endif
		gosub doRandom
	endselect
	
cnt1 = cnt1 - 1
if cnt1 > 0 then goto runIt
gosub clearSys
low pClock
low pData
end
'END of Main **************************************************
'************
'Subroutines +++++++++++++++++++++++++++++++
'set litTmp to the light
'You shift High Bit first into the shift register
clearSys:
	lights = 0
	gosub doLights
	return
	
doMyPause:
	'I have a 28x2 pause is half as long
	tmp2 = myPause
	do
		pause litSpeed
		tmp2 = tmp2 - 1
		loop while tmp2 > 0
	return

doLights:
	tmp1 = shiftCnt
	do
		litTmp = tmp1
		gosub checkLights
		if litTmp = 1 then
			high pData
			else
				low pData
			endif
'Depending on your system speed you may need delays.
'Set them in constants. Uncomment them here.
		'pauseus Set_Data
		high pClock
		'pauseus Set_Clock
		low pClock
		tmp1 = tmp1 - 1
		loop while tmp1 > 0
	return

'Shift in bits for Binary Numbers
'Low bit is Right most bit
'01 = 1, 11 = 3(1+2 = 3)
doLights2:
	tmp1 = 1
	do
		litTmp = tmp1
		gosub checkLights
		if litTmp = 1 then
			high pData
			else
				low pData
			endif
		'pauseus Set_Data
		high pClock
		'pauseus Set_Clock
		low pClock
		tmp1 = tmp1 + 1
		loop while tmp1 <= shiftCnt 
	return

doLarson:
	litTmp = cnt2
	gosub setLights
	gosub doLights
	pause litSpeed
	'CheckLights in doLights uses litTmp
	litTmp = cnt2
	gosub clearLights

	if useMreset = 1 then
		low pM_Reset
		else
			gosub doLights
		endif

	if backShift = 0 then
		cnt2 = cnt2 + 1
		if cnt2 > 8 then
			backShift = 1
			cnt2 = 7
			endif
		else
			cnt2 = cnt2 - 1
			if cnt2 = 0 then
				backShift = 0
				cnt2 = 2
				endif
		endif
		
	if useMreset = 1 then
		high pM_Reset
		endif
		
	return

doBinaryNs:
	lights = cnt2
	gosub doLights2
	gosub doMyPause
	
	if cnt2 = 32 then
		gosub doMyPause
		endif
		
	if useMreset = 1 then
		low pM_Reset
		else
			gosub doLights2
		endif
	
	cnt2 = cnt2 + 1
	if cnt2 > 32 then
		cnt2 = 1
		endif

	if useMreset = 1 then
		high pM_Reset
		endif
		
	return

doRandom:
	random myTime
	lights = myLowTime
	gosub doLights2
	gosub doMyPause
	
	if useMreset = 1 then
		low pM_Reset
		else
			lights = 0
			gosub doLights2
		endif
	
	pause 20
	if useMreset = 1 then
		high pM_Reset
		endif
		
	return

setLights:
	select case litTmp
		case 1
			lit1 = 1
		case 2
			lit2 = 1
		case 3
			lit3 = 1
		case 4
			lit4 = 1
		case 5
			lit5 = 1
		case 6
			lit6 = 1
		case 7
			lit7 = 1
		case 8
			lit8 = 1
		endselect
	return
	
clearLights:
	select case LitTmp
		case 1
			lit1 = 0
		case 2
			lit2 = 0
		case 3
			lit3 = 0
		case 4
			lit4 = 0
		case 5
			lit5 = 0
		case 6
			lit6 = 0
		case 7
			lit7 = 0
		case 8
			lit8 = 0
		endselect
	return

checkLights:
	select case litTmp
		case 1
			litTmp = lit1
		case 2
			litTmp = lit2
		case 3
			litTmp = lit3
		case 4
			litTmp = lit4
		case 5
			litTmp = lit5
		case 6
			litTmp = lit6
		case 7
			litTmp = lit7
		case 8
			litTmp = lit8
		endselect
	return
